import argparse
import fileinput
import logging

import musicpd
from mpdab.exceptions import InvalidItemException, MpdabException
from mpdab.mpdutils import (
    enqueue,
    get_all_audiobooks,
    get_audiobook_items,
    get_item,
    store_playback_bookmark,
    switch_audiobook,
)

_LOG = logging.getLogger(__name__)


def do_add(args: argparse.Namespace, client: musicpd.MPDClient) -> None:
    song_uris = [ln.strip() for ln in fileinput.input(files=args.filenames)]
    client.command_list_ok_begin()
    for idx, song_uri in enumerate(song_uris, start=1):
        client.sticker_set("song", song_uri, "audiobook", args.name)
        client.sticker_set("song", song_uri, "sequence", str(idx))
    client.command_list_end()


def do_list(args: argparse.Namespace, client: musicpd.MPDClient) -> None:
    print("\n".join(get_all_audiobooks(client)))


def do_continue(args: argparse.Namespace, client: musicpd.MPDClient) -> None:
    items = get_audiobook_items(client, args.name)
    enqueue(client, items, resume=True, autoplay=False)


def do_previous(args: argparse.Namespace, client: musicpd.MPDClient) -> None:
    switch_audiobook(client, -1)


def do_next(args: argparse.Namespace, client: musicpd.MPDClient) -> None:
    switch_audiobook(client, +1)


def do_watch(args: argparse.Namespace, client: musicpd.MPDClient) -> None:
    while True:
        events = client.idle()
        _LOG.debug("Received events from MPD: %s", events)
        if "player" in events:
            client.command_list_ok_begin()
            client.currentsong()
            client.status()
            song, status = client.command_list_end()
            song_uri = song.get("file")
            if not song_uri:
                continue

            try:
                item = get_item(client, song_uri)
            except InvalidItemException:
                _LOG.debug("Current song does not belong to an audiobook, ignoring.")
                continue

            position = status.get("elapsed")
            if position:
                store_playback_bookmark(client, item, position)
